;	===============================================
;	RGB Led-Leiste mit 10 LEDs
;	===============================================
;	Version 1.00 (10.10.2004)
;
;	System arbeitet mit 8,0 MHz
;
;	*** Timer ***:
;	Timer 0 Peak hold Timer fr VU-Meter Modus
;	Timer 1 arbeitet mit Voteiler 64 und TopWert 6250 => genaues 0,05s Timing
;	Timer 2 zustndig fr LED Helligkeit mit Vorteiler 8 =>	125kHz LED frequenz
;
;	USART asynchron mit 38,4k (UBRR = 12)
;
;	*** Ports ***:
;	PortA-Belegung:
;					LED 1-8 Blau
;			
;	PortB-Belegung:
;					LED 1-8 Rot
;			
;	PortC-Belegung:
;					LED 1-8 Grn
;			
;	PortD-Belegung:		
;					Bit 0 - UART RXD
;					Bit 1 - UART TXD
;					Bit 2+3 - LED 9+10 Blau
;					Bit 4+5 - LED 9+10 Grn
;					Bit 6+7 - LED 9+10 Rot

;
;	Konstanten
;	==========
;
.equ	LED_HellCycleLeng = 7

.equ	Timer1_Top	= 6250

;** UART commands
.equ	UCOM_Reset_Device	= 0b11110001
.equ	UCOM_Start_Bootload = 0b11110010
.equ	UCOM_Goto_VUMode	= 0b11110011
.equ	UCOM_Goto_PMode		= 0b11110100
.equ	UCOM_Goto_DMode		= 0b11110101

.equ	UCOM_DM_LEDSet		= 0b00110001	; followed by 30 bytes of LED Data
.equ	UCOM_DM_Flush		= 0b00110010	; 30 bytes nicht speichern falls nicht bereit war

.equ	UCOM_Return_Unknown		= 0b11100001
.equ	UCOM_Return_Boot_Rdy	= 0b11100010
.equ	UCOM_Return_Boot_RdyNP	= 0b11100011
.equ	UCOM_Return_DM_PrevNC	= 0b11100100


;** Program Control Bits
	;# Bit 0-5 Main Messages
.equ	PC_Start_Bootload	= 0
.equ	PC_Goto_VUMode		= 1
.equ	PC_Goto_PMode		= 2
.equ	PC_Goto_DMode		= 3
.equ	PC_PM_NextLEDSet	= 4
.equ	PC_DM_NewLEDSet		= 5

;** Mode Options and Current Mode
	;# Bit 0-3 Curent Program Mode
.equ	MC_VUMode			= 0
.equ	MC_PMode			= 1
.equ	MC_DMode			= 2
	;# Bit 6 Prev Calc in Progress don't use X,Z register
.equ	MC_DM_Busy			= 6
	;# Bit 7 Switch Led-Control-Memory Pointer (0 -> Block 1 # 1 -> Block 2)
.equ	MC_MemPointer		= 7


;
;	Register
;	========
;
.def	ProgCtrlReg	= r19
.def	ModeOptions	= r20
.def	LED_HellPosOffset = r21

.def	ProgTimeL		= r0
.def	ProgTimeH		= r1
.def	ProgTime_MaxL	= r2
.def	ProgTime_MaxH	= r3
.def	NextEventTimeL	= r4
.def	NextEventTimeH	= r5

.def	CyclePointerL	= r6
.def	CyclePointerH	= r7

.def	EventAnzahl		= r8
.def	NextEvent		= r9
.def	NextEventPointerL	= r10
.def	NextEventPointerH	= r11

.def	VUMeterWert		= r12
.def	VUMeterPeak		= r13


.def	Temp	= r16	; Zum Speichern Temporrer Werte
.def	Temp2	= r17
.def	Counter	= r18
.def	Andreg	= r22
.def	Andreg2	= r23

.def	CurCom	= r24	; Speichert den per RS-232 bermittelten Befehl solange er ausgefhrt wird (bzw. Jumpmap Offset, 0xFF fr kein befehl)
.def	CurByte	= r25	; Zuletzt per RS-232 bermitteltes Byte
.def	Addnullreg	= r14	; Reg mit wert 0

;== Reg 26-31 reserviert fr Speicheroperationen (X,Y,Z)


.include "m16def.inc"

.dseg
LED_Helligkeits_Cycle: .byte 10 * 8 * 4
VU_Current_Color: .byte 4
DM_New_LEDSet: .byte 30

.cseg
.org 0
;
;	=====================================
;	Interrupt Vektoren fr normalen Modus
;	=====================================
;
reset:	rjmp	start		;Programm-Beginn
nop
exint1:	reti				;wird nicht genutzt
nop
exint0:	reti				;wird nicht genutzt
nop
t2comp:	reti				;wird nicht genutzt
nop
t2ovf:	rjmp	PMode_LEDPWM ;Helligkeitssteuerung der LEDs
nop
t1capt:	reti				;wird nicht genutzt
nop
t1coma:	rjmp	PMode_Click	;0,05s Click
nop
t1comb:	reti				;wird nicht genutzt
nop
t1ovf:	reti				;wird nicht genutzt
nop
t0ovf:	rjmp	VUMode_TOut	;Zustndig fr Peak-Hold / Auto fall off in VU-Modus
nop
spistc:	reti				;wird nicht genutzt
nop
usarxc:	rjmp	UART_RX		;Daten empfangen per USB
nop
usadre:	reti				;wird nicht genutzt
nop
usatxc:	reti				;wird nicht genutzt
nop
adccon:	reti				;wird nicht genutzt
nop
eerdy:	reti				;wird nicht genutzt
nop
anacom:	reti				;wird nicht genutzt
nop
twi:	reti				;wird nicht genutzt
nop
exint2: reti				;wird nicht genutzt
nop
t0comp:	reti				;wird nicht genutzt
nop
spmrdy:	reti				;wird nicht genutzt
;


;
;	=================
;	Initialisierungen
;	=================
;
start:
	;== Set Stack ==
	ldi		Temp, HIGH(RAMEND)
	out		SPH, Temp
	ldi		Temp, LOW(RAMEND)
	out		SPL, Temp

	;== Init Variables ==
	ldi		Temp, 0x00
	ldi		CurCom, 0x00
	mov		Addnullreg, Temp
	ldi		ProgCtrlReg, (1 << PC_Goto_PMode)
;	ldi		ProgCtrlReg, (1 << PC_Goto_VUMode)
	ldi		ModeOptions, 0x00

	;== Init UART ==
	ldi		Temp, 12			; Baud Rate setzen
	out		UBRRL, Temp
	ldi		Temp, 0b10011000	; Enable RX, TX and RX Interrupt
	out		UCSRB, Temp
	ldi		Temp, 0b10000110	; asynchron, no parity, 1 stop bit
	out		UCSRC, Temp

	;== Set I/O Ports ==
	ldi		Temp, 0b11111111	; *** LED outputs LED 1-8 ***
	out		DDRA, Temp
	out		DDRB, Temp
	out		DDRC, Temp
	ldi		Temp, 0b11111100	; *** LED outputs LED 9+10 ***
	out		DDRD, Temp
	ldi		Temp, 0xFF
	out		PORTA, Temp
	out		PORTB, Temp
	out		PORTC, Temp
	out		PORTD, Temp

	;== Prepare Timer 0 ==
	ldi		Temp, 0b00000000 
	out		TCNT0, Temp
	ldi		Temp, 0b00000000 
	out		TCCR0, Temp

	;== Prepare Timer 1 ==
	ldi		Temp, HIGH(Timer1_Top)	;Zero Counter
	ldi		Temp2, LOW(Timer1_Top)
	out		OCR1AH, Temp
	out		OCR1AL, Temp2

	ldi		Temp, 0b00000000
	out		TCCR1A, Temp
	ldi		Temp, 0b00000000
	out		TCCR1B, Temp

;	ldi		Temp, 0b00001011 		;CTC Mode (OCR1A = Top) vorteiler 64
;	out		TCCR1B, Temp

	ldi		Temp, 0b00000000	;Zero Counter
	ldi		Temp2, 0b00000000
	out		TCNT1H, Temp
	out		TCNT1L, Temp2

	;== Prepare Timer 2 ==
	ldi		Temp, 0b00000000 
	out		TCNT2, Temp
	ldi		Temp, 0b00000000 
	out		TCCR2, Temp

	;== Set Interupts ==
	ldi		Temp, 0b01010001	;Timer 0 + 2 overflow, Timer 1 compare Match A interupt enable
	out		TIMSK, Temp
	sei		;Enable global Interrup


;
;	=============
;	Hauptschleife
;	=============
;
main:
	;**** Event Handler ****
	;## Bootloader Starten ##
	sbrc	ProgCtrlReg, PC_Start_Bootload
	jmp		bootload_init
	;## Wechseln zu VU-Meter Modus (Winamp) ##
	sbrc	ProgCtrlReg, PC_Goto_VUMode
	rcall	VUMode_init
	;## Wechseln zu Programm Modus (LED sequenz abspielen) ##
	sbrc	ProgCtrlReg, PC_Goto_PMode
	rcall	PMode_init
	;## Wechseln zu Direkt Modus (LEDs werden direkt von PC geschaltet) ##
	sbrc	ProgCtrlReg, PC_Goto_DMode
	rcall	DMode_init
	;## Vorbereiten der nchsten LED konstellation (Nur PMode) ##
	sbrc	ProgCtrlReg, PC_PM_NextLEDSet
	rcall	PMode_PrepareLedSet
	;## Vorbereiten der neuen LED konstellation (Nur DMode) ##
	sbrc	ProgCtrlReg, PC_DM_NewLEDSet
	rcall	DMode_NewLedSet

	
	;## Nichts zu tun -> Loop
	rjmp	main


;
;	=============================
;	VU-Meter Modus Initialisieren
;	=============================
;
VUMode_init:
	;** Neuen Modus in ProgCtrlReg eintragen
	andi	ProgCtrlReg, 0b00000001
	ldi		ModeOptions, (1 << MC_VUMode)
	ldi		CurCom, 0x00

	;** Timer Stoppen
	ldi		Temp, 0b00000000 
	out		TCCR1B, Temp
	out		TCCR2, Temp
	out		TCCR0, Temp
	ldi		Temp, 0xFF 
	out		PORTB, Temp
	out		PORTC, Temp
	out		PORTD, Temp

	;** Vorbereiten
	ldi		Temp, 0x00
	mov		ProgTimeL, Temp
	ldi		Temp, 0x09				;; <== Testing
	mov		VUMeterWert, Temp
	mov		VUMeterPeak, Temp

	;** Timer0 Starten
	rcall	VUMode_Out

	ldi		Temp, 0b00000011 
	out		TCCR0, Temp

	ret


;
;	==============================
;	VU-Meter Modus Anzeige Routine
;	==============================
;
VUMode_Out:
	mov		Temp, VUMeterWert
	ldi		Temp2, 0xFF

VUMode_Out_CPZero:
	cpi		Temp, 0x00
	breq	VUMode_Out_Weiter
	dec		Temp
	lsl		Temp2
	rjmp	VUMode_Out_CPZero

VUMode_Out_Weiter:
	out		PORTA, Temp2
	ret


;
;	=========================================================
;	VU-Meter Modus T0 Ovf zur verarbeitung von Peak Hold usw.
;	=========================================================
;
VUMode_TOut:
	inc		ProgTimeL

	ldi		Temp, 0x10
	cp		ProgTimeL, Temp
	breq	VUMode_TOut_Click

	reti

VUMode_TOut_Click:
	ldi		Temp, 0x00
	mov		ProgTimeL, Temp

	cp		VUMeterWert, AddNullReg
	brne	VUMode_TOut_Dec

	reti

VUMode_TOut_Dec:
	dec		VUMeterWert
	rcall	VUMode_Out

	reti



;
;	============================
;	Program Modus Initialisieren
;	============================
;
PMode_init:
	;** Neuen Modus in ProgCtrlReg eintragen
	andi	ProgCtrlReg, 0b00000001
	ldi		ModeOptions, (1 << MC_PMode)
	ldi		LED_HellPosOffset, 0x00
	ldi		CurCom, 0x00

	;** Timer Stoppen
	ldi		Temp, 0b00000000 
	out		TCCR1B, Temp
	out		TCCR2, Temp
	out		TCCR0, Temp

	;** Aktuelle Zeit und auf Null
	ldi		Temp, 0x00
	mov		ProgTimeL, Temp
	mov		ProgTimeH, Temp

	;** Program Optionen laden
	ldi		ZH, high(ProgramOptions<<1)		; Z pointer auf Program Optionen setzen
	ldi		ZL, low(ProgramOptions<<1)
	lpm		ProgTime_MaxL, Z+				; Program Optionen einlesen
	lpm		ProgTime_MaxH, Z+
	lpm		EventAnzahl, Z+

	;** Beginne Laden von erstem Event
	ldi		Temp, 0x00
	mov		NextEvent, Temp
	ldi		Temp, high(ProgramDaten<<1)
	mov		NextEventPointerH, Temp
	ldi		Temp, low(ProgramDaten<<1)
	mov		NextEventPointerL, Temp

	rcall	PMode_PrepareLedSet

	;** Prfen ob erstes Event bei t0
	ldi		Temp, 0x00
	cp		NextEventTimeL, Temp
	cpc		NextEventTimeH, Temp
	breq	PMode_init_FirstEvent
	rjmp	PMode_init_Weiter

PMode_init_FirstEvent:
	;** Aktuelle Event Helligkeit einstellen und Befehl zum Event nachladen
	sbr		ModeOptions, (1 << MC_MemPointer)
	sbr		ProgCtrlReg, (1 << PC_PM_NextLEDSet)

PMode_init_Weiter:
	;**	Timer starten
	ldi		Temp, 0b00000010 
	out		TCCR2, Temp
	ldi		Temp, 0b00001011 		;CTC Mode (OCR1A = Top) vorteiler 64
	out		TCCR1B, Temp

	ret


;
;	====================================
;	Nchstes Helligkeits Set vorbereiten
;	====================================
;
PMode_PrepareLedSet:
	cbr		ProgCtrlReg, (1 << PC_PM_NextLEDSet)

	;* Datenpointer Setzen
	mov		ZH, NextEventPointerH		; Z pointer auf Program Daten setzen
	mov		ZL, NextEventPointerL
	lpm		NextEventTimeL, Z+			; Event Zeitpunkt laden
	lpm		NextEventTimeH, Z+

	;* Derzeit inaktiven Helligkeits Cycle zuweisen (Backbuffer)
	ldi		Temp, low(LED_Helligkeits_Cycle)	;Set Startadress of Helligkeits Cycle
	mov		CyclePointerL, Temp
	ldi		Temp, high(LED_Helligkeits_Cycle)
	mov		CyclePointerH, Temp

	sbrc	ModeOptions, MC_MemPointer
	rjmp	PMode_PrepareLedSetM_Weiter

	;* Falls erster aktiv Pointer auf zweiten setzten
	ldi		Temp, 32
	add		CyclePointerL, Temp
	adc		CyclePointerH, Addnullreg

PMode_PrepareLedSetM_Weiter:
	;* Laden von Rot 1-8
	mov		XL, CyclePointerL	; Set Startadress of Helligkeits Cycle PORTB (Rot)
	mov		XH, CyclePointerH
	adiw	XH:XL, 1			
	rcall	PMode_PrepareLedSet_LED18	;Prepare Blau 1-6

	;* Laden von Grn 1-8
	mov		XL, CyclePointerL	; Set Startadress of Helligkeits Cycle PORTC (Grn)
	mov		XH, CyclePointerH
	adiw	XH:XL, 2
	adiw	ZH:ZL, 1			; Set to Programmemory beginn of Grn
	rcall	PMode_PrepareLedSet_LED18	;Prepare Blau 1-6

	;* Laden von Blau 1-8
	mov		XL, CyclePointerL	; Set Startadress of Helligkeits Cycle PORTA (Blau)
	mov		XH, CyclePointerH
	adiw	ZH:ZL, 1			; Set to Programmemory beginn of Blau
	rcall	PMode_PrepareLedSet_LED18	;Prepare Blau 1-6

	;* Nachladen von Leds 9+10
	mov		XL, CyclePointerL	; Set Startadress of Helligkeits Cycle PORTD (LEDs 9+10)
	mov		XH, CyclePointerH
	adiw	XH:XL, 3
	sbiw	ZH:ZL, 8			; Set to Programmemory last Byte of Rot
	rcall	PMode_PrepareLedSet_LED910

	;* Next Event erhhen
	inc		NextEvent
	cp		NextEvent, EventAnzahl		;Letztes event verarbeitet?
	breq	PMode_PrepareLedSetM_Restart

	;* Letztes Event noch nicht erreicht Event counter um eins erhhen
	ldi		Temp, 14
	add		NextEventPointerL, Temp
	adc		NextEventPointerH, AddNullReg

	ret

	;* Und es geht wieder mit Event 0 los
PMode_PrepareLedSetM_Restart:
	ldi		Temp, 0x00
	mov		NextEvent, Temp
	ldi		Temp, high(ProgramDaten<<1)
	mov		NextEventPointerH, Temp
	ldi		Temp, low(ProgramDaten<<1)
	mov		NextEventPointerL, Temp

	ret


PMode_PrepareLedSet_LED18:		;************ 1. Byte von Farbe bei ausgewhltem Event in Z -- 1. Set von Helligkeit in Cycle des Zugehrigen Ports in X ************
	;** Start with first led set in cycle
	ldi		Temp2, 0xFF	; Helligkeit als "Aus" initialisieren

	lpm		Temp, Z+	; 1. Byte von Ausgewhlter farbe
	sbrc	Temp, 0x00			; LED 1 Helligkeit
	cbr		Temp2, (1 << 0)
	sbrc	Temp, 0x03			; LED 2 Helligkeit
	cbr		Temp2, (1 << 1)
	sbrc	Temp, 0x06			; LED 3 Helligkeit
	cbr		Temp2, (1 << 2)

	lpm		Temp, Z+	; 2. Byte von Ausgewhlter farbe
	sbrc	Temp, 0x01			; LED 4 Helligkeit
	cbr		Temp2, (1 << 3)
	sbrc	Temp, 0x04			; LED 5 Helligkeit
	cbr		Temp2, (1 << 4)
	sbrc	Temp, 0x07			; LED 6 Helligkeit
	cbr		Temp2, (1 << 5)

	lpm		Temp, Z+	; 3. Byte von Ausgewhlter farbe
	sbrc	Temp, 0x02			; LED 7 Helligkeit
	cbr		Temp2, (1 << 6)
	sbrc	Temp, 0x05			; LED 8 Helligkeit
	cbr		Temp2, (1 << 7)

	st		X, Temp2			; Store first set of Hell von ausgewhltem PORT

	;** Proceed with Second + Third led set in cycle
	sbiw	ZH:ZL, 3	; wieder 1. Byte von Ausgewhlter farbe

	ldi		Temp2, 0xFF	;; Helligkeit als "Aus" initialisieren

	lpm		Temp, Z+	; 1. Byte von Ausgewhlter farbe
	sbrc	Temp, 0x01			; LED 1 Helligkeit
	cbr		Temp2, (1 << 0)
	sbrc	Temp, 0x04			; LED 2 Helligkeit
	cbr		Temp2, (1 << 1)
	sbrc	Temp, 0x07			; LED 3 Helligkeit
	cbr		Temp2, (1 << 2)

	lpm		Temp, Z+	; 2. Byte von Ausgewhlter farbe
	sbrc	Temp, 0x02			; LED 4 Helligkeit
	cbr		Temp2, (1 << 3)
	sbrc	Temp, 0x05			; LED 5 Helligkeit
	cbr		Temp2, (1 << 4)

	lpm		Temp, Z+	; 3. Byte von Ausgewhlter farbe
	sbrc	Temp, 0x00			; LED 6 Helligkeit
	cbr		Temp2, (1 << 5)
	sbrc	Temp, 0x03			; LED 7 Helligkeit
	cbr		Temp2, (1 << 6)
	sbrc	Temp, 0x06			; LED 8 Helligkeit
	cbr		Temp2, (1 << 7)

	adiw	XH:XL, 4	; nchster cycle von Helligkeit von ausgewhltem PORT
	st		X, Temp2
	adiw	XH:XL, 4	; nchster cycle von Helligkeit von ausgewhltem PORT
	st		X, Temp2

	;** Last four led sets in cycle
	sbiw	ZH:ZL, 3	; wieder 1. Byte von Ausgewhlter farbe

	ldi		Temp2, 0xFF	; Helligkeit als "Aus" initialisieren

	lpm		Temp, Z+	; 1. Byte von Ausgewhlter farbe
	sbrc	Temp, 0x02			; LED 1 Helligkeit
	cbr		Temp2, (1 << 0)
	sbrc	Temp, 0x05			; LED 2 Helligkeit
	cbr		Temp2, (1 << 1)

	lpm		Temp, Z+	; 2. Byte von Ausgewhlter farbe
	sbrc	Temp, 0x00			; LED 3 Helligkeit
	cbr		Temp2, (1 << 2)
	sbrc	Temp, 0x03			; LED 4 Helligkeit
	cbr		Temp2, (1 << 3)
	sbrc	Temp, 0x06			; LED 5 Helligkeit
	cbr		Temp2, (1 << 4)

	lpm		Temp, Z+	; 3. Byte von Ausgewhlter farbe
	sbrc	Temp, 0x01			; LED 6 Helligkeit
	cbr		Temp2, (1 << 5)
	sbrc	Temp, 0x04			; LED 7 Helligkeit
	cbr		Temp2, (1 << 6)
	sbrc	Temp, 0x07			; LED 8 Helligkeit
	cbr		Temp2, (1 << 7)

	adiw	XH:XL, 4	; nchster cycle von Helligkeit von ausgewhltem PORT
	st		X, Temp2
	adiw	XH:XL, 4	; nchster cycle von Helligkeit von ausgewhltem PORT
	st		X, Temp2
	adiw	XH:XL, 4	; nchster cycle von Helligkeit von ausgewhltem PORT
	st		X, Temp2
	adiw	XH:XL, 4	; nchster cycle von Helligkeit von ausgewhltem PORT
	st		X, Temp2

	ret


PMode_PrepareLedSet_LED910:		;************ Reste von Helligkeit (LED 9+10) verarbeiten ************
	;** Start with first led set in cycle
	ldi		Temp2, 0xFC	; Helligkeit als "Aus" initialisieren

	lpm		Temp, Z		; 1. Reste von Rote
	sbrc	Temp, 0x00			; LED 9 Helligkeit
	cbr		Temp2, (1 << 6)
	sbrc	Temp, 0x03			; LED 10 Helligkeit
	cbr		Temp2, (1 << 7)

	adiw	ZH:ZL, 4	; nchste Farbe
	lpm		Temp, Z		; 2. Reste von Grn
	sbrc	Temp, 0x00			; LED 9 Helligkeit
	cbr		Temp2, (1 << 4)
	sbrc	Temp, 0x03			; LED 10 Helligkeit
	cbr		Temp2, (1 << 5)

	adiw	ZH:ZL, 4	; nchste Farbe
	lpm		Temp, Z		; 3. Reste von Blau
	sbrc	Temp, 0x00			; LED 9 Helligkeit
	cbr		Temp2, (1 << 2)
	sbrc	Temp, 0x03			; LED 10 Helligkeit
	cbr		Temp2, (1 << 3)

	st		X, Temp2			; Store first set of Hell von ausgewhltem PORT

	;** Proceed with Second + Third led set in cycle
	ldi		Temp2, 0xFC	;; Helligkeit als "Aus" initialisieren

	sbiw	ZH:ZL, 8	; wieder reste von rot
	lpm		Temp, Z		; 1. Reste von Rote
	sbrc	Temp, 0x01			; LED 9 Helligkeit
	cbr		Temp2, (1 << 6)
	sbrc	Temp, 0x04			; LED 10 Helligkeit
	cbr		Temp2, (1 << 7)

	adiw	ZH:ZL, 4	; nchste Farbe
	lpm		Temp, Z		; 2. Reste von Grn
	sbrc	Temp, 0x01			; LED 9 Helligkeit
	cbr		Temp2, (1 << 4)
	sbrc	Temp, 0x04			; LED 10 Helligkeit
	cbr		Temp2, (1 << 5)

	adiw	ZH:ZL, 4	; nchste Farbe
	lpm		Temp, Z		; 3. Reste von Blau
	sbrc	Temp, 0x01			; LED 9 Helligkeit
	cbr		Temp2, (1 << 2)
	sbrc	Temp, 0x04			; LED 10 Helligkeit
	cbr		Temp2, (1 << 3)
	adiw	XH:XL, 4	; nchster cycle von Helligkeit von ausgewhltem PORT
	st		X, Temp2
	adiw	XH:XL, 4	; nchster cycle von Helligkeit von ausgewhltem PORT
	st		X, Temp2

	;** Last four led sets in cycle
	ldi		Temp2, 0xFC	; Helligkeit als "Aus" initialisieren

	sbiw	ZH:ZL, 8	; wieder reste von rot
	lpm		Temp, Z		; 1. Reste von Rote
	sbrc	Temp, 0x02			; LED 9 Helligkeit
	cbr		Temp2, (1 << 6)
	sbrc	Temp, 0x05			; LED 10 Helligkeit
	cbr		Temp2, (1 << 7)

	adiw	ZH:ZL, 4	; nchste Farbe
	lpm		Temp, Z		; 2. Reste von Grn
	sbrc	Temp, 0x02			; LED 9 Helligkeit
	cbr		Temp2, (1 << 4)
	sbrc	Temp, 0x05			; LED 10 Helligkeit
	cbr		Temp2, (1 << 5)

	adiw	ZH:ZL, 4	; nchste Farbe
	lpm		Temp, Z		; 3. Reste von Blau
	sbrc	Temp, 0x02			; LED 9 Helligkeit
	cbr		Temp2, (1 << 2)
	sbrc	Temp, 0x05			; LED 10 Helligkeit
	cbr		Temp2, (1 << 3)

	adiw	XH:XL, 4	; nchster cycle von Helligkeit von ausgewhltem PORT
	st		X, Temp2
	adiw	XH:XL, 4	; nchster cycle von Helligkeit von ausgewhltem PORT
	st		X, Temp2
	adiw	XH:XL, 4	; nchster cycle von Helligkeit von ausgewhltem PORT
	st		X, Temp2
	adiw	XH:XL, 4	; nchster cycle von Helligkeit von ausgewhltem PORT
	st		X, Temp2

	ret


;
;	========================================
;	Nchster Programclick + auf Event Prfen
;	========================================
;
PMode_Click:
	push	Temp
	;** Programm zeit Hochzhlen
	ldi		Temp, 1
	add		ProgTimeL, Temp
	adc		ProgTimeH, AddNullReg

	;** Prfen ob Programmende Erreicht
	cp		ProgTimeL, ProgTime_MaxL	; Compare low byte
	cpc		ProgTimeH, ProgTime_MaxH	; Compare high byte
	brsh	PMode_Click_ZeroProgTime	; Branch if Programm Ende erreicht
	rjmp	PMode_Click_Event			; Weiter mit Event berprfung

PMode_Click_ZeroProgTime:
	ldi		Temp, 0x00			; Zeit auf Null und weiter mit Prfen auf Event
	mov		ProgTimeL, Temp
	mov		ProgTimeH, Temp

	;** Prfen ob neues Event
PMode_Click_Event:
	cp		ProgTimeL, NextEventTimeL	; Compare low byte
	cpc		ProgTimeH, NextEventTimeH	; Compare high byte
	breq	PMode_Click_NextEvent

	pop		Temp
	reti

PMode_Click_NextEvent:
	;** Aktuelle Event Helligkeit Cycle Tauschen und Befehl zum Event nachladen
	sbrs	ModeOptions, MC_MemPointer
	rjmp	PMode_Click_NextEvent_Cycl2

PMode_Click_NextEvent_Cycl1:
	cbr		ModeOptions, (1 << MC_MemPointer)
	rjmp	PMode_Click_NextEvent_Weiter

PMode_Click_NextEvent_Cycl2:
	sbr		ModeOptions, (1 << MC_MemPointer)

PMode_Click_NextEvent_Weiter:
	sbr		ProgCtrlReg, (1 << PC_PM_NextLEDSet)

	pop		Temp
	reti


;
;	=============================
;	Helligkeitssteuerung der LEDs
;	=============================
;
PMode_LEDPWM:
	push	Temp

	;* Derzeit aktiven Helligkeits Cycle zuweisen (Frontbuffer)
	ldi		YL, low(LED_Helligkeits_Cycle)		;Set Startadress of Helligkeits Cycle 1
	ldi		YH, high(LED_Helligkeits_Cycle)

	sbrs	ModeOptions, MC_MemPointer
	rjmp	PMode_LEDPWM_Weiter

	;* Falls zweiter aktiv Pointer auf zweiten setzten
	adiw	YH:YL, 32

PMode_LEDPWM_Weiter:
	add		YL, LED_HellPosOffset		; 16-Bit Add offset
	adc		YH, Addnullreg		

	;** Output prepared LED on/off values
	ld		Temp, Y+
	out		PORTA, Temp
	ld		Temp, Y+
	out		PORTB, Temp
	ld		Temp, Y+
	out		PORTC, Temp
	ld		Temp, Y+
	out		PORTD, Temp

	cpi		LED_HellPosOffset, (LED_HellCycleLeng - 1) * 4
	breq	PMode_LEDPWM_Nulloffset

	;** Offset um 4 erhhen wenn ende noch nciht erreicht
	ldi		Temp, 0x04
	add		LED_HellPosOffset, Temp
	pop		Temp
	reti

PMode_LEDPWM_Nulloffset:
	ldi		LED_HellPosOffset, 0x00
	pop		Temp
	reti

;
;	===========================
;	Direkt Modus Initialisieren
;	===========================
;
DMode_init:
	;** Neuen Modus in ProgCtrlReg eintragen
	andi	ProgCtrlReg, 0b00000001
	ldi		ModeOptions, (1 << MC_DMode)
	ldi		CurCom, 0x00

	;** Timer Stoppen
	ldi		Temp, 0b00000000 
	out		TCCR1B, Temp
	out		TCCR2, Temp
	out		TCCR0, Temp
	ldi		Temp, 0xFF 
	out		PORTA, Temp
	out		PORTB, Temp
	out		PORTC, Temp
	out		PORTD, Temp

	;** Vorbereiten
	;* Derzeitiges Set auf alles aus
	ldi		Temp, 0x00
	ldi		Temp2, 0xFF
	ldi		XL, low(LED_Helligkeits_Cycle)	;Set Startadress of New Farben
	ldi		XH, high(LED_Helligkeits_Cycle)
DMode_init_ledsoff:
	st		X+, Temp2
	inc		Temp
	cpi		Temp, 0x20
	brlo	DMode_init_ledsoff

	;** Timer2 Starten
	ldi		Temp, 0x00
	out		TCNT2, Temp
	ldi		Temp, 0b00000010 
	out		TCCR2, Temp

	ret

;
;	================================
;	Nchstes Helligkeits Set Starten
;	================================
;
DMode_NewLedSet:
	cbr		ProgCtrlReg, (1 << PC_DM_NewLEDSet)
	sbr		ModeOptions, (1 << MC_DM_Busy)

	;* Derzeit inaktiven Helligkeits Cycle zuweisen (Backbuffer)
	ldi		Temp, low(LED_Helligkeits_Cycle)	;Set Startadress of Helligkeits Cycle
	mov		CyclePointerL, Temp
	ldi		Temp, high(LED_Helligkeits_Cycle)
	mov		CyclePointerH, Temp

	sbrc	ModeOptions, MC_MemPointer
	rjmp	DMode_NewLedSet_Weiter

	;* Falls erster aktiv Pointer auf zweiten setzten
	ldi		Temp, 32
	add		CyclePointerL, Temp
	adc		CyclePointerH, Addnullreg

DMode_NewLedSet_Weiter:
	;* Laden von Rot 1-8
	mov		XL, CyclePointerL	; Set Startadress of Helligkeits Cycle PORTB (Rot)
	mov		XH, CyclePointerH
	adiw	XH:XL, 1			

	ldi		ZL, low(DM_New_LEDSet)	;Set Startadress of New Farben
	ldi		ZH, high(DM_New_LEDSet)

	;* LED 1-8 Rot helligkeiten einlesen
	rcall	DMode_NewLedSet_LED18

	;* Laden von Grn 1- 8
	adiw	ZH:ZL, 10	;Set Adress to Helligkeiten von Grn

	mov		XL, CyclePointerL	; Set Startadress of Helligkeits Cycle PORTC (Grn)
	mov		XH, CyclePointerH
	adiw	XH:XL, 2			

	;* LED 1-8 Grn helligkeiten einlesen
	rcall	DMode_NewLedSet_LED18

	;* Laden von Blau 1- 8
	adiw	ZH:ZL, 10	;Set Adress to Helligkeiten von Blau

	mov		XL, CyclePointerL	; Set Startadress of Helligkeits Cycle PORTA (Blau)
	mov		XH, CyclePointerH

	;* LED 1-8 Blau helligkeiten einlesen
	rcall	DMode_NewLedSet_LED18

	;* Laden von Reste 9-10
	ldi		ZL, low(DM_New_LEDSet)	;Set Startadress of Rot9
	ldi		ZH, high(DM_New_LEDSet)
	adiw	ZH:ZL, 8

	mov		XL, CyclePointerL	; Set Startadress of Helligkeits Cycle PORTD (Reste)
	mov		XH, CyclePointerH
	adiw	XH:XL, 3

	;* LED 9-10 Reste helligkeiten einlesen
	rcall	DMode_NewLedSet_LED910


	;** Aktuelle Helligkeit Cycle Tauschen
	sbrs	ModeOptions, MC_MemPointer
	rjmp	DMode_NewLedSet_Cycl2
DMode_NewLedSet_Cycl1:
	cbr		ModeOptions, (1 << MC_MemPointer)
	rjmp	DMode_NewLedSet_Ende

DMode_NewLedSet_Cycl2:
	sbr		ModeOptions, (1 << MC_MemPointer)

DMode_NewLedSet_Ende:
	cbr		ModeOptions, (1 << MC_DM_Busy)
	ret


DMode_NewLedSet_LED18:		;************ helligkeiten fr ersten 8 leds von (Port) einlesen ************
	;** Start with first led set in cycle
	ldi		Andreg2, 0x01
	rcall	DMode_NewLedSet_Hell18
	rcall	DMode_NewLedSet_Hell18
	rcall	DMode_NewLedSet_Hell18
	rcall	DMode_NewLedSet_Hell18
	rcall	DMode_NewLedSet_Hell18
	rcall	DMode_NewLedSet_Hell18
	rcall	DMode_NewLedSet_Hell18
	rcall	DMode_NewLedSet_Hell18

	ret

DMode_NewLedSet_Hell18:		;************ helligkeits nr fr entsprechneds einlesen ************
	ldi		Temp2, 0xFF	; Helligkeit als "Aus" initialisieren
	ldi		Counter, 0x00
	ldi		Andreg, 0xFE

DMode_NewLedSet_LED18_H1:
	ld		Temp, Z+			; Byte von Ausgewhlter farbe 1. LED
	and		Temp, Andreg2
	cpi		Temp, 0x00
	breq	DMode_NewLedSet_LED18_H2
	and		Temp2, Andreg		; Bit lschen wenn Hell bei dieser Led gesetzt
DMode_NewLedSet_LED18_H2:
	sec
	rol		Andreg
	inc		counter
	cpi		counter, 8
	brlo	DMode_NewLedSet_LED18_H1
	st		X, Temp2			; Store first set of Hell von ausgewhltem PORT
	sbiw	ZH:ZL, 8			; Wieder auf 1. Led setzen
	adiw	XH:XL, 4			; Set to nex Hell von Port
	lsl		Andreg2

	ret


DMode_NewLedSet_LED910:		;************ helligkeiten fr led 9+10 (PortD) einlesen ************
	;** Start with first led set in cycle
	ldi		Andreg2, 0x01
	rcall	DMode_NewLedSet_Hell910
	rcall	DMode_NewLedSet_Hell910
	rcall	DMode_NewLedSet_Hell910
	rcall	DMode_NewLedSet_Hell910
	rcall	DMode_NewLedSet_Hell910
	rcall	DMode_NewLedSet_Hell910
	rcall	DMode_NewLedSet_Hell910
	rcall	DMode_NewLedSet_Hell910

	ret

DMode_NewLedSet_Hell910:		;************ helligkeits nr fr entsprechneds einlesen ************
	ldi		Temp2, 0xFC	; Helligkeit als "Aus" initialisieren

	;**	Reste von Rot einlesen
	ldi		Counter, 0x00
	ldi		Andreg, 0b10111111
DMode_NewLedSet_LED910_H1:
	ld		Temp, Z+			; Byte von Rot9+10
	and		Temp, Andreg2
	cpi		Temp, 0x00
	breq	DMode_NewLedSet_LED910_H2
	and		Temp2, Andreg		; Bit lschen wenn Hell bei dieser Led gesetzt
DMode_NewLedSet_LED910_H2:
	sec
	rol		Andreg
	inc		counter
	cpi		counter, 2
	brlo	DMode_NewLedSet_LED910_H1

	;** Reste con Grn einlesen
	adiw	ZH:ZL, 8			; Auf Grn9+10
	ldi		Counter, 0x00
	ldi		Andreg, 0b11101111
DMode_NewLedSet_LED910_H3:
	ld		Temp, Z+			; Byte von Grn9+10
	and		Temp, Andreg2
	cpi		Temp, 0x00
	breq	DMode_NewLedSet_LED910_H4
	and		Temp2, Andreg		; Bit lschen wenn Hell bei dieser Led gesetzt
DMode_NewLedSet_LED910_H4:
	sec
	rol		Andreg
	inc		counter
	cpi		counter, 2
	brlo	DMode_NewLedSet_LED910_H3

	;** Reste con Blau einlesen
	adiw	ZH:ZL, 8			; Auf Blau9+10
	ldi		Counter, 0x00
	ldi		Andreg, 0b11111011
DMode_NewLedSet_LED910_H5:
	ld		Temp, Z+			; Byte von Blau9+10
	and		Temp, Andreg2
	cpi		Temp, 0x00
	breq	DMode_NewLedSet_LED910_H6
	and		Temp2, Andreg		; Bit lschen wenn Hell bei dieser Led gesetzt
DMode_NewLedSet_LED910_H6:
	sec
	rol		Andreg
	inc		counter
	cpi		counter, 2
	brlo	DMode_NewLedSet_LED910_H5

	st		X, Temp2			; Store first set of Hell von ausgewhltem PORT
	adiw	XH:XL, 4			; Set to next Hell von Port
	sbiw	ZH:ZL, 22			; Reset to Rot9+10
	lsl		Andreg2

	ret


;
;	=====================
;	Serielle Daten von PC
;	=====================
;
UART_RX:
	in		CurByte, UDR

	;** Falls bertragung von Daten Luft
	cpi		CurCom, UCOM_DM_LEDSet
	breq	UART_RX_DMode

	cpi		CurCom, UCOM_DM_Flush
	breq	UART_RX_DMode_Flush
	
	;** Befehls Behandlung
	cpi		CurByte, UCOM_Start_Bootload
	breq	UART_RX_Start_Bootload

	cpi		CurByte, UCOM_Goto_VUMode
	breq	UART_RX_Goto_VUMode

	cpi		CurByte, UCOM_Goto_PMode
	breq	UART_RX_Goto_PMode

	cpi		CurByte, UCOM_Goto_DMode
	breq	UART_RX_Goto_DMode

	cpi		CurByte, UCOM_DM_LEDSet
	breq	UART_RX_DM_LEDSet_Init

	sbrc	ModeOptions, MC_VUMode
	rjmp	UART_RX_VUMode

	;** Ansonsten unbekannter Befehl nachfragen und fertig
	ldi		Temp, UCOM_Return_Unknown
	out		UDR, Temp
	reti

	;**** Handle Start Bootload request
UART_RX_Start_Bootload:
	sbr		ProgCtrlReg, (1 << PC_Start_Bootload)
	reti

	;**** Handle Goto VU-Mode request
UART_RX_Goto_VUMode:
	sbr		ProgCtrlReg, (1 << PC_Goto_VUMode)
	reti

	;**** Handle Goto Program Mode request
UART_RX_Goto_PMode:
	sbr		ProgCtrlReg, (1 << PC_Goto_PMode)
	reti

	;**** Handle Goto Direkt Mode request
UART_RX_Goto_DMode:
	sbr		ProgCtrlReg, (1 << PC_Goto_DMode)
	reti

	;**** == Byte in VU-Modus empfangen ==
UART_RX_VUMode:
	andi	CurByte, 0b00001111
	mov		VUMeterWert, CurByte

	reti

	;**** == Vorbereiten zum empfangen von Bytes fr LEDSet ==
UART_RX_DM_LEDSet_Init:
	sbrc	ModeOptions, MC_DM_Busy
	rjmp	UART_RX_DM_LEDSet_Init_BusyErr

	mov		CurCom, CurByte
	;** Vorbereiten
	;* Speicher Pointer auf start
	ldi		Counter, 0x00	; byte recived counter
	ldi		XL, low(DM_New_LEDSet)	;Set Startadress of New Farben
	ldi		XH, high(DM_New_LEDSet)

	reti

UART_RX_DM_LEDSet_Init_BusyErr:
	ldi		Temp, UCOM_Return_DM_PrevNC
	out		UDR, Temp
	ldi		CurCom, UCOM_DM_Flush
	ldi		Counter, 0x00	; byte recived counter

	reti

	;**** == Byte in Direkt-Modus empfangen ==
UART_RX_DMode:
	st		X+, CurByte
	inc		Counter
	cpi		Counter, 0x1E
	brlo	UART_RX_DMode_Ende

	;** 30 bytes empfangen
	ldi		CurCom, 0x00
	sbr		ProgCtrlReg, (1 << PC_DM_NewLEDSet)

UART_RX_DMode_Ende:
	reti

	;**** == Bytes nicht speichern falls nicht bereit war ==
UART_RX_DMode_Flush:
	inc		Counter
	cpi		Counter, 0x30
	brlo	UART_RX_DMode_Flush_Ende
	reti
UART_RX_DMode_Flush_Ende:
	ldi		CurCom, 0x00
	reti





ProgramOptions: 	.DW 130, 26
ProgramDaten:		.DW 0  , 0b1011111011001000, 0b0000000001100000, 0b1011111011001000, 0b0000000000000000, 0b0000000000000000, 0b0000100001100000
ProgramDaten1:		.DW 5  , 0b0001011111011001, 0b0000000000001100, 0b0001011111011001, 0b0000000000000000, 0b0000000000000000, 0b0001100100001100
ProgramDaten2:		.DW 10 , 0b1000001011111011, 0b0000000000000001, 0b0000001011111011, 0b0000000000000000, 0b1000000000000000, 0b0011101100100001
ProgramDaten3:		.DW 15 , 0b0011000001011111, 0b0000000000000000, 0b0000000001011111, 0b0000000000000000, 0b0011000000000000, 0b0001111101100100
ProgramDaten4:		.DW 20 , 0b0000011000001011, 0b0000000000000000, 0b0000000000001011, 0b0000000000000000, 0b1000011000000000, 0b0000101111101100
ProgramDaten5:		.DW 25 , 0b0000000011000001, 0b0000000000000000, 0b0000000000000001, 0b0000000000000000, 0b1000000011000000, 0b0000000101111101

ProgramDaten6:		.DW 30 , 0b0000000000011000, 0b0000000000000000, 0b0000000000000000, 0b0000100000000000, 0b1011001000011000, 0b0000000000101111
ProgramDaten7:		.DW 35 , 0b0000000000000011, 0b0000000000000000, 0b0000000000000000, 0b0001100100000000, 0b1111011001000011, 0b0000000000000101
ProgramDaten8:		.DW 40 , 0b0000000000000000, 0b0000000000000000, 0b0000000000000000, 0b0011101100100000, 0b1011111011001000, 0b0000000000000000
ProgramDaten9:		.DW 45 , 0b0000000000000000, 0b0000000000000000, 0b0000000000000000, 0b0001111101100100, 0b0101111101100100, 0b0000000000000000
ProgramDaten10:		.DW 50 , 0b0000000000000000, 0b0000000000000000, 0b1000000000000000, 0b0000101111101100, 0b0000101111101100, 0b0000000000000000
ProgramDaten11:		.DW 55 , 0b0000000000000000, 0b0000000000000000, 0b1001000000000000, 0b0000000101111101, 0b0000000101111100, 0b0000000000000000

ProgramDaten12:		.DW 60 , 0b0000000000000000, 0b0000100000000000, 0b1011001000000000, 0b0000000000101111, 0b0000000000101100, 0b0000000000000000
ProgramDaten13:		.DW 65 , 0b0000000000000000, 0b0001100100000000, 0b1111011001000000, 0b0000000000000101, 0b0000000000000100, 0b0000000000000000
ProgramDaten14:		.DW 70 , 0b0000000000000000, 0b0011101100100000, 0b1011111011001000, 0b0000000000000000, 0b0000000000000000, 0b0000000000000000
ProgramDaten15:		.DW 75 , 0b0000000000000000, 0b0001111101100100, 0b0001011111011001, 0b0000000000000000, 0b0000000000000000, 0b0000000000000000
ProgramDaten16:		.DW 80 , 0b1000000000000000, 0b0000101111101100, 0b0000001011111011, 0b0000000000000000, 0b0000000000000000, 0b0000000000000000
ProgramDaten17:		.DW 85 , 0b1001000000000000, 0b0000000101111101, 0b0000000001011111, 0b0000000000000000, 0b0000000000000000, 0b0000000000000000

ProgramDaten18:		.DW 90 , 0b1011001000000000, 0b0000100000101111, 0b0000000000001011, 0b0000100000000000, 0b0000000000000000, 0b0000000000000000
ProgramDaten19:		.DW 95 , 0b1111011001000000, 0b0001100100000101, 0b0000000000000001, 0b0001100100000000, 0b0000000000000000, 0b0000000000000000
ProgramDaten20:		.DW 100, 0b0111111011001000, 0b0011101100100000, 0b0000000000000000, 0b0011101100100000, 0b0000000000000000, 0b0000000000000000
ProgramDaten21:		.DW 105, 0b0001011111011001, 0b0001111101100100, 0b0000000000000000, 0b0001111101100100, 0b0000000000000000, 0b0000000000000000
ProgramDaten22:		.DW 110, 0b1000001011111011, 0b0000101111101100, 0b1000000000000000, 0b0000101111101100, 0b0000000000000000, 0b0000000000000000
ProgramDaten23:		.DW 115, 0b1001000001011111, 0b0000000101111101, 0b1001000000000000, 0b0000000101111101, 0b0000000000000000, 0b0000000000000000
	
ProgramDaten24:		.DW 120, 0b1011001000001011, 0b0001100000101111, 0b1011001000000000, 0b0000000000101111, 0b0000000000000000, 0b0001100000000000
ProgramDaten25:		.DW 125, 0b1111011001000001, 0b0000001100000101, 0b1111011001000000, 0b0000000000000101, 0b0000000000000000, 0b0000001100000000


.org SECONDBOOTSTART + 0x02
;
;	========================================
;	Interrupt Vektoren fr Programmier Modus
;	========================================
;
prog_exint1:	reti				;wird nicht genutzt
nop
prog_exint0:	reti				;wird nicht genutzt
nop
prog_t2comp:	reti				;wird nicht genutzt
nop
prog_t2ovf:		reti				;wird nicht genutzt
nop
prog_t1capt:	reti				;wird nicht genutzt
nop
prog_t1coma:	reti				;wird nicht genutzt
nop
prog_t1comb:	reti				;wird nicht genutzt
nop
prog_t1ovf:		reti				;wird nicht genutzt
nop
prog_t0ovf:		reti				;wird nicht genutzt
nop
prog_spistc:	reti				;wird nicht genutzt
nop
prog_usarxc:	jmp	UART_RX			;Daten empfangen per USB
nop
prog_usadre:	reti				;wird nicht genutzt
nop
prog_usatxc:	reti				;wird nicht genutzt
nop
prog_adccon:	reti				;wird nicht genutzt
nop
prog_eerdy:		reti				;wird nicht genutzt
nop
prog_anacom:	reti				;wird nicht genutzt
nop
prog_twi:		reti				;wird nicht genutzt
nop
prog_exint2:	reti				;wird nicht genutzt
nop
prog_t0comp:	reti				;wird nicht genutzt
nop
prog_spmrdy:	reti				;wird nicht genutzt
;


;
;	========================================================
;	Bootloader zum Programmieren von LED-Sequenzen ins Flash
;	========================================================
;
bootload_init:
	;*** Timer Stoppen und Interupts neu einstellen
	cli
	ldi		Temp, 0b00000000 
	out		TCCR0, Temp
	out		TCCR1B, Temp
	out		TCCR2, Temp

	ldi		Temp, (1<<IVCE)
	out		GICR, Temp
	ldi		Temp, (1<<IVSEL)
	out		GICR, Temp
	sei
	;*** Interupts wieder an
	andi	ProgCtrlReg, 0b10000000



	jmp		main

















































